# SpotiViz

## Project description and features

SpotiViz is a Python program that uses "Top 100 Most Streamed Songs on Spotify" to retrieve and analyze track information, including audio features such as danceability, energy, and loudness. The program provides a GUI interface that allows users to search for tracks by artist, album, or track name, and select specific tracks to analyze. Users can also select which audio features to visualize and analyze, such as beats.per.minute, energy or danceability. The processed data is then displayed using various charts and graphs, such as scatter plots, histograms, and bar charts, to display the audio features of each track.

The project will be implemented using Python, with the following main components:

- Data Loading and Processing: The dataset will be loaded into a Python data structure (e.g., Pandas dataframe) and processed to extract relevant features and information.

- Graphical User Interface (GUI): A GUI will be developed using the Tkinter library to provide an interactive interface for users to explore the dataset. The GUI will allow users to select and filter data based on different criteria (e.g., artist, energy, danceability), and display visualizations of the data.

- Data Visualization: The processed data will be visualized using various graphical representations, such as bar charts, scatterplots, and network graphs. These visualizations will provide insights into the most popular songs and artists, as well as user listening behavior and preferences.

# Data sources
"Top 100 Most Streamed Songs on Spotify" from https://www.kaggle.com/datasets/pavan9065/top-100-most-streamed-songs-on-spotify

# Running the application, including libraries used and installation
`Tkinter`: a standard Python GUI library used for creating and managing windows, buttons, labels, etc. in the user interface.

`Matplotlib`: a popular data visualization library in Python used for creating various types of charts and graphs.

`Random`: a module that provides functions for generating random numbers and data.

`Numpy`: a library for numerical computing in Python. It provides functions for working with arrays, matrices, and other numerical data types.

`Pandas`: for data frame and data visualization.

`Networkx`: a library for working with graphs and networks in Python. It provides functions for creating, manipulating, and analyzing graphs, including algorithms for shortest.

`Sklearn`: a library for machine learning in Python. It provides functions for clustering, regression, classification, and other machine learning tasks.

## Requirements
- Python 3.x installed on the system.
- virtual env
 
`pip install virtualenv`

# How to run code in terminal
`cd SpotiVizProgram`

`python main.py`

Or you can install by `requirement.txt`

# OOP design
* Here is class diagram of the program.
![ClassDiagram](SpotiVizProgram/assets/ClassDiagram.png)
* Here is Sequence diagram of the program.
![Sequence diagram1](SpotiVizProgram/assets/Sq1Diagram.png)
![Sequence diagram2](SpotiVizProgram/assets/dSq2Diagram.png)


# Design patterns used
-	Facade - MVC (Model-View-Controller) pattern: Using this pattern helps to divide the program into clear and distinct parts, making it easier to manage the program.
# Graph problem modeling and algorithm
I use the shortest-path algorithm to recommend songs. The program will recommend 5 songs that are similar to the selected target song. There will be two recommendation functions in the program that use this algorithm, with one using the sklearn library to speed up the process.

